@@ -13,12 +13,17 @@ from __future__ import annotations
 import inspect
 import warnings
 from abc import ABC, abstractmethod
-from typing import Callable, List, Optional
+from typing import Callable, List, Optional, Union
 
 import torch
+from botorch.exceptions.errors import UnsupportedError
+from botorch.models.model import Model
 from botorch.posteriors.gpytorch import GPyTorchPosterior, scalarize_posterior
 from botorch.posteriors.posterior import Posterior
+from botorch.sampling import IIDNormalSampler, MCSampler
 from botorch.utils import apply_constraints
+from gpytorch.distributions import MultitaskMultivariateNormal, MultivariateNormal
+from gpytorch.lazy import lazify
 from torch import Tensor
 from torch.nn import Module
 
@@ -61,6 +66,10 @@ class PosteriorTransform(Module, ABC):
         pass  # pragma: no cover
 
 
+# import DeterministicModel after PosteriorTransform to avoid circular import
+from botorch.models.deterministic import DeterministicModel  # noqa
+
+
 class ScalarizedPosteriorTransform(PosteriorTransform):
     r"""An affine posterior transform for scalarizing multi-output posteriors.
 
@@ -131,6 +140,110 @@ class ScalarizedObjective(ScalarizedPosteriorTransform, AcquisitionObjective):
         super().__init__(weights=weights, offset=offset)
 
 
+class ExpectationPosteriorTransform(PosteriorTransform):
+    r"""Transform the `batch x (q * n_w) x m` posterior into a `batch x q x m`
+    posterior of the expectation. The expectation is calculated over each
+    consecutive `n_w` block of points in the posterior.
+
+    This is intended for use with `InputPerturbation` or `AppendFeatures` for
+    optimizing the expectation over `n_w` points. This should not be used when
+    there are constraints present, since this does not take into account
+    the feasibility of the objectives.
+
+    Note: This is different than `ScalarizedPosteriorTransform` in that
+    this operates over the q-batch dimension.
+    """
+
+    def __init__(self, n_w: int, weights: Optional[Tensor] = None) -> None:
+        r"""A posterior transform calculating the expectation over the q-batch
+        dimension.
+
+        Args:
+            n_w: The number of points in the q-batch of the posterior to compute
+                the expectation over. This corresponds to the size of the
+                `feature_set` of `AppendFeatures` or the size of the `perturbation_set`
+                of `InputPerturbation`.
+            weights: An optional `n_w x m`-dim tensor of weights. Can be used to
+                compute a weighted expectation. Weights are normalized before use.
+        """
+        super().__init__()
+        if weights is not None:
+            if weights.dim() != 2 or weights.shape[0] != n_w:
+                raise ValueError("`weights` must be a tensor of size `n_w x m`.")
+            if torch.any(weights < 0):
+                raise ValueError("`weights` must be non-negative.")
+        else:
+            weights = torch.ones(n_w, 1)
+        # Normalize the weights.
+        weights = weights / weights.sum(dim=0)
+        self.register_buffer("weights", weights)
+        self.n_w = n_w
+
+    def evaluate(self, Y: Tensor) -> Tensor:
+        r"""Evaluate the expectation of a set of outcomes.
+
+        Args:
+            Y: A `batch_shape x (q * n_w) x m`-dim tensor of outcomes.
+
+        Returns:
+            A `batch_shape x q x m`-dim tensor of expectation outcomes.
+        """
+        batch_shape, m = Y.shape[:-2], Y.shape[-1]
+        weighted_Y = Y.view(*batch_shape, -1, self.n_w, m) * self.weights.to(Y)
+        return weighted_Y.sum(dim=-2)
+
+    def forward(self, posterior: GPyTorchPosterior) -> GPyTorchPosterior:
+        r"""Compute the posterior of the expectation.
+
+        Args:
+            posterior: An `m`-outcome joint posterior over `q * n_w` points.
+
+        Returns:
+            An `m`-outcome joint posterior over `q` expectations.
+        """
+        org_mvn = posterior.mvn
+        if getattr(org_mvn, "_interleaved", False):
+            raise UnsupportedError(
+                "`ExpectationPosteriorTransform` does not support "
+                "interleaved posteriors."
+            )
+        # Initialize the weight matrix of shape compatible with the mvn.
+        org_event_shape = org_mvn.event_shape
+        batch_shape = org_mvn.batch_shape
+        q = org_event_shape[0] // self.n_w
+        m = 1 if len(org_event_shape) == 1 else org_event_shape[-1]
+        tkwargs = {"device": org_mvn.loc.device, "dtype": org_mvn.loc.dtype}
+        weights = torch.zeros(q * m, q * self.n_w * m, **tkwargs)
+        # Make sure self.weights has the correct dtype/device and shape.
+        self.weights = self.weights.to(org_mvn.loc).expand(self.n_w, m)
+        # Fill in the non-zero entries of the weight matrix.
+        # We want each row to have non-zero weights for the corresponding
+        # `n_w` sized diagonal. The `m` outcomes are not interleaved.
+        for i in range(q * m):
+            weights[i, self.n_w * i : self.n_w * (i + 1)] = self.weights[:, i // q]
+        # Trasform the mean.
+        new_loc = (
+            (weights @ org_mvn.loc.unsqueeze(-1))
+            .view(*batch_shape, m, q)
+            .transpose(-1, -2)
+        )
+        # Transform the covariance matrix.
+        org_cov = (
+            org_mvn.lazy_covariance_matrix
+            if org_mvn.islazy
+            else org_mvn.covariance_matrix
+        )
+        new_cov = weights @ (org_cov @ weights.t())
+        if m == 1:
+            new_mvn = MultivariateNormal(new_loc.squeeze(-1), lazify(new_cov))
+        else:
+            # Using MTMVN since we pass a single loc and covar for all `m` outputs.
+            new_mvn = MultitaskMultivariateNormal(
+                new_loc, lazify(new_cov), interleaved=False
+            )
+        return GPyTorchPosterior(mvn=new_mvn)
+
+
 class MCAcquisitionObjective(Module, ABC):
     r"""Abstract base class for MC-based objectives.
 
@@ -328,7 +441,7 @@ class ConstrainedMCObjective(GenericMCObjective):
         self,
         objective: Callable[[Tensor, Optional[Tensor]], Tensor],
         constraints: List[Callable[[Tensor], Tensor]],
-        infeasible_cost: float = 0.0,
+        infeasible_cost: Union[Tensor, float] = 0.0,
         eta: float = 1e-3,
     ) -> None:
         r"""Feasibility-weighted objective.
@@ -349,7 +462,7 @@ class ConstrainedMCObjective(GenericMCObjective):
         """
         super().__init__(objective=objective)
         self.constraints = constraints
-        self.eta = eta
+        self.register_buffer("eta", torch.as_tensor(eta))
         self.register_buffer("infeasible_cost", torch.as_tensor(infeasible_cost))
 
     def forward(self, samples: Tensor, X: Optional[Tensor] = None) -> Tensor:
@@ -373,3 +486,68 @@ class ConstrainedMCObjective(GenericMCObjective):
             infeasible_cost=self.infeasible_cost,
             eta=self.eta,
         )
+
+
+class LearnedObjective(MCAcquisitionObjective):
+    r"""Learned preference objective constructed from a preference model.
+
+    For input `samples`, it samples each individual sample again from the latent
+    preference posterior distribution using `pref_model` and return the posterior mean.
+
+    Example:
+        >>> train_X = torch.rand(2, 2)
+        >>> train_comps = torch.LongTensor([[0, 1]])
+        >>> pref_model = PairwiseGP(train_X, train_comps)
+        >>> learned_pref_obj = LearnedObjective(pref_model)
+        >>> samples = sampler(posterior)
+        >>> objective = learned_pref_obj(samples)
+    """
+
+    def __init__(
+        self,
+        pref_model: Model,
+        sampler: Optional[MCSampler] = None,
+    ):
+        r"""Learned preference objective constructed from a preference model.
+
+        Args:
+            pref_model: A BoTorch model, which models the latent preference/utility
+                function. Given an input tensor of size
+                `sample_size x batch_shape x N x d`, its `posterior` method should
+                return a `Posterior` object with single outcome representing the
+                utility values of the input.
+            sampler: Sampler for the preference model to account for uncertainty in
+                preferece when calculating the objective; it's not the one used
+                in MC acquisition functions. If None,
+                it uses `IIDNormalSampler(num_samples=1)`.
+        """
+        super().__init__()
+        self.pref_model = pref_model
+        if isinstance(pref_model, DeterministicModel):
+            assert sampler is None
+            self.sampler = None
+        else:
+            if sampler is None:
+                self.sampler = IIDNormalSampler(num_samples=1)
+            else:
+                self.sampler = sampler
+
+    def forward(self, samples: Tensor, X: Optional[Tensor] = None) -> Tensor:
+        r"""Sample each element of samples.
+
+        Args:
+            samples: A `sample_size x batch_shape x N x d`-dim Tensors of
+                samples from a model posterior.
+
+        Returns:
+            A `(sample_size * num_samples) x batch_shape x N`-dim Tensor of
+            objective values sampled from utility posterior using `pref_model`.
+        """
+        post = self.pref_model.posterior(samples)
+        if isinstance(self.pref_model, DeterministicModel):
+            # return preference posterior mean
+            return post.mean.squeeze(-1)
+        else:
+            # return preference posterior sample mean
+            samples = self.sampler(post).squeeze(-1)
+            return samples.reshape(-1, *samples.shape[2:])  # batch_shape x N
