@@ -22,7 +22,7 @@ see also [Hong2014review]_.
 
 from abc import ABC, abstractmethod
 from math import ceil
-from typing import Optional
+from typing import List, Optional, Union
 
 import torch
 from botorch.acquisition.objective import MCAcquisitionObjective
@@ -47,18 +47,20 @@ class RiskMeasureMCObjective(MCAcquisitionObjective, ABC):
     def __init__(
         self,
         n_w: int,
-        weights: Optional[Tensor] = None,
+        weights: Optional[Union[List[float], Tensor]] = None,
     ) -> None:
         r"""Transform the posterior samples to samples of a risk measure.
 
         Args:
             n_w: The size of the `w_set` to calculate the risk measure over.
-            weights: An optional `m`-dim tensor of weights for scalarizing
+            weights: An optional `m`-dim tensor or list of weights for scalarizing
                 multi-output samples before calculating the risk measure.
         """
         super().__init__()
         self.n_w = n_w
-        self.weights = weights
+        self.register_buffer(
+            "weights", torch.as_tensor(weights) if weights is not None else None
+        )
 
     def _prepare_samples(self, samples: Tensor) -> Tensor:
         r"""Prepare samples for risk measure calculations by scalarizing and
@@ -77,6 +79,7 @@ class RiskMeasureMCObjective(MCAcquisitionObjective, ABC):
                 "Multi-output samples require `weights` for scalarization!"
             )
         if self.weights is not None:
+            self.weights = self.weights.to(samples)
             samples = samples @ self.weights
         else:
             samples = samples.squeeze(-1)
@@ -116,14 +119,14 @@ class CVaR(RiskMeasureMCObjective):
         self,
         alpha: float,
         n_w: int,
-        weights: Optional[Tensor] = None,
+        weights: Optional[Union[List[float], Tensor]] = None,
     ) -> None:
         r"""Transform the posterior samples to samples of a risk measure.
 
         Args:
             alpha: The risk level, float in `(0.0, 1.0]`.
             n_w: The size of the `w_set` to calculate the risk measure over.
-            weights: An optional `m`-dim tensor of weights for scalarizing
+            weights: An optional `m`-dim tensor or list of weights for scalarizing
                 multi-objective samples before calculating the risk measure.
         """
         super().__init__(n_w=n_w, weights=weights)
@@ -166,14 +169,14 @@ class VaR(CVaR):
         self,
         alpha: float,
         n_w: int,
-        weights: Optional[Tensor] = None,
+        weights: Optional[Union[List[float], Tensor]] = None,
     ) -> None:
         r"""Transform the posterior samples to samples of a risk measure.
 
         Args:
             alpha: The risk level, float in `(0.0, 1.0]`.
             n_w: The size of the `w_set` to calculate the risk measure over.
-            weights: An optional `m`-dim tensor of weights for scalarizing
+            weights: An optional `m`-dim tensor or list of weights for scalarizing
                 multi-objective samples before calculating the risk measure.
         """
         super().__init__(n_w=n_w, alpha=alpha, weights=weights)
@@ -225,3 +228,31 @@ class WorstCase(RiskMeasureMCObjective):
         """
         prepared_samples = self._prepare_samples(samples)
         return prepared_samples.min(dim=-1).values
+
+
+class Expectation(RiskMeasureMCObjective):
+    r"""The expectation risk measure.
+
+    For unconstrained problems, we recommend using the `ExpectationPosteriorTransform`
+    instead. `ExpectationPosteriorTransform` directly transforms the posterior
+    distribution over `q * n_w` to a posterior of `q` expectations, significantly
+    reducing the cost of posterior sampling as a result.
+    """
+
+    def forward(self, samples: Tensor, X: Optional[Tensor] = None) -> Tensor:
+        r"""Calculate the expectation corresponding to the given samples.
+        This calculates the expectation / mean / average of each `n_w` samples
+        across the q-batch dimension. If `self.weights` is given, the samples
+        are scalarized across the output dimension before taking the expectation.
+
+        Args:
+            samples: A `sample_shape x batch_shape x (q * n_w) x m`-dim tensor of
+                posterior samples. The q-batches should be ordered so that each
+                `n_w` block of samples correspond to the same input.
+            X: A `batch_shape x q x d`-dim tensor of inputs. Ignored.
+
+        Returns:
+            A `sample_shape x batch_shape x q`-dim tensor of expectation samples.
+        """
+        prepared_samples = self._prepare_samples(samples)
+        return prepared_samples.mean(dim=-1)
