@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -10,7 +10,9 @@ Candidate generation utilities.
 
 from __future__ import annotations
 
-from typing import Any, Dict, List, Optional, Tuple, Type, Union
+import warnings
+from functools import partial
+from typing import Any, Callable, Dict, List, NoReturn, Optional, Tuple, Type, Union
 
 import numpy as np
 import torch
@@ -20,6 +22,8 @@ from botorch.optim.parameter_constraints import (
     _arrayify,
     make_scipy_bounds,
     make_scipy_linear_constraints,
+    make_scipy_nonlinear_inequality_constraints,
+    NLC_TOL,
 )
 from botorch.optim.stopping import ExpMAStoppingCriterion
 from botorch.optim.utils import _filter_kwargs, columnwise_clamp, fix_features
@@ -35,6 +39,7 @@ def gen_candidates_scipy(
     upper_bounds: Optional[Union[float, Tensor]] = None,
     inequality_constraints: Optional[List[Tuple[Tensor, Tensor, float]]] = None,
     equality_constraints: Optional[List[Tuple[Tensor, Tensor, float]]] = None,
+    nonlinear_inequality_constraints: Optional[List[Callable]] = None,
     options: Optional[Dict[str, Any]] = None,
     fixed_features: Optional[Dict[int, Optional[float]]] = None,
 ) -> Tuple[Tensor, Tensor]:
@@ -54,6 +59,11 @@ def gen_candidates_scipy(
         equality constraints: A list of tuples (indices, coefficients, rhs),
             with each tuple encoding an inequality constraint of the form
             `\sum_i (X[indices[i]] * coefficients[i]) = rhs`.
+        nonlinear_inequality_constraints: A list of callables with that represent
+            non-linear inequality constraints of the form `callable(x) >= 0`. Each
+            callable is expected to take a `(num_restarts) x q x d`-dim tensor as
+            an input and return a `(num_restarts) x q`-dim tensor with the
+            constraint values. The constraints will later be passed to SLSQP.
         options: Options used to control the optimization including "method"
             and "maxiter". Select method for `scipy.minimize` using the
             "method" key. By default uses L-BFGS-B for box-constrained problems
@@ -85,15 +95,24 @@ def gen_candidates_scipy(
     """
     options = options or {}
 
-    # REDUCED is used indicate if we are optimizing over a reduced domain dimension
-    # after considering fixed_features.
-    # REDUCED mode if fixed_features is not None except for when fixed_features.values()
-    # contains None and linear constraints are passed.
-    REDUCED = fixed_features is not None
-    if inequality_constraints or equality_constraints:
-        REDUCED = REDUCED and (None not in fixed_features.values())
+    # if there are fixed features we may optimize over a domain of lower dimension
+    reduced_domain = False
+    if fixed_features:
+        # TODO: We can support fixed features, see Max's comment on D33551393. We can
+        # consider adding this at a later point.
+        if nonlinear_inequality_constraints:
+            raise NotImplementedError(
+                "Fixed features are not supported when non-linear inequality "
+                "constraints are given."
+            )
+        # if there are no constraints things are straightforward
+        if not (inequality_constraints or equality_constraints):
+            reduced_domain = True
+        # if there are we need to make sure features are fixed to specific values
+        else:
+            reduced_domain = None not in fixed_features.values()
 
-    if REDUCED:
+    if reduced_domain:
         _no_fixed_features = _remove_fixed_features_from_optimization(
             fixed_features=fixed_features,
             acquisition_function=acquisition_function,
@@ -103,7 +122,6 @@ def gen_candidates_scipy(
             inequality_constraints=inequality_constraints,
             equality_constraints=equality_constraints,
         )
-
         # call the routine with no fixed_features
         clamped_candidates, batch_acquisition = gen_candidates_scipy(
             initial_conditions=_no_fixed_features.initial_conditions,
@@ -125,7 +143,7 @@ def gen_candidates_scipy(
     )
 
     shapeX = clamped_candidates.shape
-    x0 = _arrayify(clamped_candidates.view(-1))
+    x0 = clamped_candidates.view(-1)
     bounds = make_scipy_bounds(
         X=initial_conditions, lower_bounds=lower_bounds, upper_bounds=upper_bounds
     )
@@ -135,7 +153,8 @@ def gen_candidates_scipy(
         equality_constraints=equality_constraints,
     )
 
-    def f(x):
+    def f_np_wrapper(x: np.ndarray, f: Callable):
+        """Given a torch callable, compute value + grad given a numpy array."""
         if np.isnan(x).any():
             raise RuntimeError(
                 f"{np.isnan(x).sum()} elements of the {x.size} element array "
@@ -149,7 +168,7 @@ def gen_candidates_scipy(
             .requires_grad_(True)
         )
         X_fix = fix_features(X, fixed_features=fixed_features)
-        loss = -acquisition_function(X_fix).sum()
+        loss = f(X_fix).sum()
         # compute gradient w.r.t. the inputs (does not accumulate in leaves)
         gradf = _arrayify(torch.autograd.grad(loss, X)[0].contiguous().view(-1))
         if np.isnan(gradf).any():
@@ -163,9 +182,27 @@ def gen_candidates_scipy(
         fval = loss.item()
         return fval, gradf
 
+    if nonlinear_inequality_constraints:
+        # Make sure `batch_limit` is 1 for now.
+        if not (len(shapeX) == 3 and shapeX[:2] == torch.Size([1, 1])):
+            raise ValueError(
+                "`batch_limit` must be 1 when non-linear inequality constraints "
+                "are given."
+            )
+        constraints += make_scipy_nonlinear_inequality_constraints(
+            nonlinear_inequality_constraints=nonlinear_inequality_constraints,
+            f_np_wrapper=f_np_wrapper,
+            x0=x0,
+        )
+    x0 = _arrayify(x0)
+
+    def f(x):
+        return -acquisition_function(x)
+
     res = minimize(
-        f,
-        x0,
+        fun=f_np_wrapper,
+        args=(f,),
+        x0=x0,
         method=options.get("method", "SLSQP" if constraints else "L-BFGS-B"),
         jac=True,
         bounds=bounds,
@@ -178,6 +215,18 @@ def gen_candidates_scipy(
         fixed_features=fixed_features,
     )
 
+    # SLSQP sometimes fails in the line search or may just fail to find a feasible
+    # candidate in which case we just return the starting point. This happens rarely,
+    # so it shouldn't be an issue given enough restarts.
+    if nonlinear_inequality_constraints and any(
+        nlc(candidates.view(-1)) < NLC_TOL for nlc in nonlinear_inequality_constraints
+    ):
+        candidates = torch.from_numpy(x0).to(candidates).reshape(shapeX)
+        warnings.warn(
+            "SLSQP failed to converge to a solution the satisfies the non-linear "
+            "constraints. Returning the feasible starting point."
+        )
+
     clamped_candidates = columnwise_clamp(
         X=candidates, lower=lower_bounds, upper=upper_bounds, raise_on_violation=True
     )
@@ -194,7 +243,7 @@ def gen_candidates_torch(
     upper_bounds: Optional[Union[float, Tensor]] = None,
     optimizer: Type[Optimizer] = torch.optim.Adam,
     options: Optional[Dict[str, Union[float, str]]] = None,
-    verbose: bool = True,
+    callback: Optional[Callable[[int, Tensor, Tensor], NoReturn]] = None,
     fixed_features: Optional[Dict[int, Optional[float]]] = None,
 ) -> Tuple[Tensor, Tensor]:
     r"""Generate a set of candidates using a `torch.optim` optimizer.
@@ -211,7 +260,9 @@ def gen_candidates_torch(
             candidate search.
         options: Options used to control the optimization. Includes
             maxiter: Maximum number of iterations
-        verbose: If True, provide verbose output.
+        callback: A callback function accepting the current iteration, loss,
+            and gradients as arguments. This function is executed after computing
+            the loss and gradients, but before calling the optimizer.
         fixed_features: This is a dictionary of feature indices to values, where
             all generated candidates will have features fixed to these values.
             If the dictionary value is None, then that feature will just be
@@ -239,12 +290,9 @@ def gen_candidates_torch(
     """
     options = options or {}
 
-    # REDUCED is used indicate if we are optimizing over a reduced domain dimension
-    # after considering fixed_features.
-    REDUCED = fixed_features is not None
-
-    if REDUCED:
-        _no_fixed_features = _remove_fixed_features_from_optimization(
+    # if there are fixed features we may optimize over a domain of lower dimension
+    if fixed_features:
+        subproblem = _remove_fixed_features_from_optimization(
             fixed_features=fixed_features,
             acquisition_function=acquisition_function,
             initial_conditions=initial_conditions,
@@ -256,26 +304,24 @@ def gen_candidates_torch(
 
         # call the routine with no fixed_features
         clamped_candidates, batch_acquisition = gen_candidates_torch(
-            initial_conditions=_no_fixed_features.initial_conditions,
-            acquisition_function=_no_fixed_features.acquisition_function,
-            lower_bounds=_no_fixed_features.lower_bounds,
-            upper_bounds=_no_fixed_features.upper_bounds,
+            initial_conditions=subproblem.initial_conditions,
+            acquisition_function=subproblem.acquisition_function,
+            lower_bounds=subproblem.lower_bounds,
+            upper_bounds=subproblem.upper_bounds,
             optimizer=optimizer,
             options=options,
-            verbose=verbose,
+            callback=callback,
             fixed_features=None,
         )
-        clamped_candidates = _no_fixed_features.acquisition_function._construct_X_full(
+        clamped_candidates = subproblem.acquisition_function._construct_X_full(
             clamped_candidates
         )
         return clamped_candidates, batch_acquisition
 
-    clamped_candidates = columnwise_clamp(
-        X=initial_conditions, lower=lower_bounds, upper=upper_bounds
-    ).requires_grad_(True)
-    bayes_optimizer = optimizer(
-        params=[clamped_candidates], lr=options.get("lr", 0.025)
-    )
+    _clamp = partial(columnwise_clamp, lower=lower_bounds, upper=upper_bounds)
+    clamped_candidates = _clamp(initial_conditions).requires_grad_(True)
+    _optimizer = optimizer(params=[clamped_candidates], lr=options.get("lr", 0.025))
+
     i = 0
     stop = False
     stopping_criterion = ExpMAStoppingCriterion(
@@ -283,28 +329,23 @@ def gen_candidates_torch(
     )
     while not stop:
         i += 1
-        loss = -acquisition_function(clamped_candidates).sum()
-        if verbose:
-            print("Iter: {} - Value: {:.3f}".format(i, -(loss.item())))
+        with torch.no_grad():
+            X = _clamp(clamped_candidates).requires_grad_(True)
 
-        def closure():
-            bayes_optimizer.zero_grad()
-            output_grad = torch.autograd.grad(loss, clamped_candidates)[0]
-            clamped_candidates.grad = output_grad
+        loss = -acquisition_function(X).sum()
+        grad = torch.autograd.grad(loss, X)[0]
+        if callback:
+            callback(i, loss, grad)
+
+        def assign_grad():
+            _optimizer.zero_grad()
+            clamped_candidates.grad = grad
             return loss
 
-        bayes_optimizer.step(closure)
-        with torch.no_grad():
-            clamped_candidates = columnwise_clamp(
-                X=clamped_candidates, lower=lower_bounds, upper=upper_bounds
-            ).requires_grad_(True)
+        _optimizer.step(assign_grad)
         stop = stopping_criterion.evaluate(fvals=loss.detach())
-    clamped_candidates = columnwise_clamp(
-        X=clamped_candidates,
-        lower=lower_bounds,
-        upper=upper_bounds,
-        raise_on_violation=True,
-    )
+
+    clamped_candidates = _clamp(clamped_candidates)
     with torch.no_grad():
         batch_acquisition = acquisition_function(clamped_candidates)
 
