@@ -10,7 +10,7 @@ Gaussian Process Regression models based on GPyTorch models.
 
 from __future__ import annotations
 
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, List, Optional, Union
 
 import torch
 from botorch import settings
@@ -19,7 +19,6 @@ from botorch.models.transforms.input import InputTransform
 from botorch.models.transforms.outcome import Log, OutcomeTransform
 from botorch.models.utils import fantasize as fantasize_flag, validate_input_scaling
 from botorch.sampling.samplers import MCSampler
-from botorch.utils.containers import TrainingData
 from gpytorch.constraints.constraints import GreaterThan
 from gpytorch.distributions.multivariate_normal import MultivariateNormal
 from gpytorch.kernels.matern_kernel import MaternKernel
@@ -32,6 +31,7 @@ from gpytorch.likelihoods.gaussian_likelihood import (
 from gpytorch.likelihoods.likelihood import Likelihood
 from gpytorch.likelihoods.noise_models import HeteroskedasticNoise
 from gpytorch.means.constant_mean import ConstantMean
+from gpytorch.means.mean import Mean
 from gpytorch.mlls.noise_model_added_loss_term import NoiseModelAddedLossTerm
 from gpytorch.models.exact_gp import ExactGP
 from gpytorch.module import Module
@@ -66,6 +66,7 @@ class SingleTaskGP(BatchedMultiOutputGPyTorchModel, ExactGP):
         train_Y: Tensor,
         likelihood: Optional[Likelihood] = None,
         covar_module: Optional[Module] = None,
+        mean_module: Optional[Mean] = None,
         outcome_transform: Optional[OutcomeTransform] = None,
         input_transform: Optional[InputTransform] = None,
     ) -> None:
@@ -78,6 +79,8 @@ class SingleTaskGP(BatchedMultiOutputGPyTorchModel, ExactGP):
                 GaussianLikelihood with inferred noise level.
             covar_module: The module computing the covariance (Kernel) matrix.
                 If omitted, use a `MaternKernel`.
+            mean_module: The mean function to be used. If omitted, use a
+                `ConstantMean`.
             outcome_transform: An outcome transform that is applied to the
                 training data during instantiation and to the posterior during
                 inference (that is, the `Posterior` obtained by calling
@@ -118,9 +121,11 @@ class SingleTaskGP(BatchedMultiOutputGPyTorchModel, ExactGP):
         else:
             self._is_custom_likelihood = True
         ExactGP.__init__(self, train_X, train_Y, likelihood)
-        self.mean_module = ConstantMean(batch_shape=self._aug_batch_shape)
+        if mean_module is None:
+            mean_module = ConstantMean(batch_shape=self._aug_batch_shape)
+        self.mean_module = mean_module
         if covar_module is None:
-            self.covar_module = ScaleKernel(
+            covar_module = ScaleKernel(
                 MaternKernel(
                     nu=2.5,
                     ard_num_dims=transformed_X.shape[-1],
@@ -136,8 +141,7 @@ class SingleTaskGP(BatchedMultiOutputGPyTorchModel, ExactGP):
                 "covar_module.raw_outputscale": -1,
                 "covar_module.base_kernel.raw_lengthscale": -3,
             }
-        else:
-            self.covar_module = covar_module
+        self.covar_module = covar_module
         # TODO: Allow subsetting of other covar modules
         if outcome_transform is not None:
             self.outcome_transform = outcome_transform
@@ -152,19 +156,6 @@ class SingleTaskGP(BatchedMultiOutputGPyTorchModel, ExactGP):
         covar_x = self.covar_module(x)
         return MultivariateNormal(mean_x, covar_x)
 
-    @classmethod
-    def construct_inputs(
-        cls, training_data: TrainingData, **kwargs: Any
-    ) -> Dict[str, Any]:
-        r"""Construct kwargs for the `Model` from `TrainingData` and other options.
-
-        Args:
-            training_data: `TrainingData` container with data for single outcome
-                or for multiple outcomes for batched multi-output case.
-            **kwargs: None expected for this class.
-        """
-        return {"train_X": training_data.X, "train_Y": training_data.Y}
-
 
 class FixedNoiseGP(BatchedMultiOutputGPyTorchModel, ExactGP):
     r"""A single-task exact GP model using fixed noise levels.
@@ -183,6 +174,7 @@ class FixedNoiseGP(BatchedMultiOutputGPyTorchModel, ExactGP):
         train_Y: Tensor,
         train_Yvar: Tensor,
         covar_module: Optional[Module] = None,
+        mean_module: Optional[Mean] = None,
         outcome_transform: Optional[OutcomeTransform] = None,
         input_transform: Optional[InputTransform] = None,
         **kwargs: Any,
@@ -194,6 +186,10 @@ class FixedNoiseGP(BatchedMultiOutputGPyTorchModel, ExactGP):
             train_Y: A `batch_shape x n x m` tensor of training observations.
             train_Yvar: A `batch_shape x n x m` tensor of observed measurement
                 noise.
+            covar_module: The module computing the covariance (Kernel) matrix.
+                If omitted, use a `MaternKernel`.
+            mean_module: The mean function to be used. If omitted, use a
+                `ConstantMean`.
             outcome_transform: An outcome transform that is applied to the
                 training data during instantiation and to the posterior during
                 inference (that is, the `Posterior` obtained by calling
@@ -227,9 +223,11 @@ class FixedNoiseGP(BatchedMultiOutputGPyTorchModel, ExactGP):
         ExactGP.__init__(
             self, train_inputs=train_X, train_targets=train_Y, likelihood=likelihood
         )
-        self.mean_module = ConstantMean(batch_shape=self._aug_batch_shape)
+        if mean_module is None:
+            mean_module = ConstantMean(batch_shape=self._aug_batch_shape)
+        self.mean_module = mean_module
         if covar_module is None:
-            self.covar_module = ScaleKernel(
+            covar_module = ScaleKernel(
                 base_kernel=MaternKernel(
                     nu=2.5,
                     ard_num_dims=transformed_X.shape[-1],
@@ -244,8 +242,7 @@ class FixedNoiseGP(BatchedMultiOutputGPyTorchModel, ExactGP):
                 "covar_module.raw_outputscale": -1,
                 "covar_module.base_kernel.raw_lengthscale": -3,
             }
-        else:
-            self.covar_module = covar_module
+        self.covar_module = covar_module
         # TODO: Allow subsetting of other covar modules
         if input_transform is not None:
             self.input_transform = input_transform
@@ -324,25 +321,6 @@ class FixedNoiseGP(BatchedMultiOutputGPyTorchModel, ExactGP):
         new_model.likelihood.noise_covar.noise = new_noise
         return new_model
 
-    @classmethod
-    def construct_inputs(
-        cls, training_data: TrainingData, **kwargs: Any
-    ) -> Dict[str, Any]:
-        r"""Construct kwargs for the `Model` from `TrainingData` and other options.
-
-        Args:
-            training_data: `TrainingData` container with data for single outcome
-                or for multiple outcomes for batched multi-output case.
-            **kwargs: None expected for this class.
-        """
-        if training_data.Yvar is None:
-            raise ValueError(f"Yvar required for {cls.__name__}.")
-        return {
-            "train_X": training_data.X,
-            "train_Y": training_data.Y,
-            "train_Yvar": training_data.Yvar,
-        }
-
 
 class HeteroskedasticSingleTaskGP(SingleTaskGP):
     r"""A single-task exact GP model using a heteroskeastic noise model.
