@@ -14,7 +14,7 @@ Gaussian Process Regression models based on GPyTorch models.
 
 from __future__ import annotations
 
-from typing import Any, Dict, Optional, Tuple
+from typing import Any, Dict, List, Optional, Tuple
 
 import torch
 from botorch.exceptions.errors import UnsupportedError
@@ -26,7 +26,7 @@ from botorch.models.kernels.linear_truncated_fidelity import (
 )
 from botorch.models.transforms.input import InputTransform
 from botorch.models.transforms.outcome import OutcomeTransform
-from botorch.utils.containers import TrainingData
+from botorch.utils.datasets import SupervisedDataset
 from gpytorch.kernels.kernel import ProductKernel
 from gpytorch.kernels.rbf_kernel import RBFKernel
 from gpytorch.kernels.scale_kernel import ScaleKernel
@@ -126,24 +126,24 @@ class SingleTaskMultiFidelityGP(SingleTaskGP):
         self.to(train_X)
 
     @classmethod
-    def construct_inputs(cls, training_data: TrainingData, **kwargs) -> Dict[str, Any]:
-        r"""Construct kwargs for the `Model` from `TrainingData` and other options.
+    def construct_inputs(
+        cls,
+        training_data: SupervisedDataset,
+        fidelity_features: List[int],
+        **kwargs,
+    ) -> Dict[str, Any]:
+        r"""Construct `Model` keyword arguments from a dict of `SupervisedDataset`.
 
         Args:
-            training_data: `TrainingData` container with data for single outcome
-                or for multiple outcomes for batched multi-output case.
-            **kwargs: Options, expected for this class:
-                - fidelity_features: List of columns of X that are fidelity parameters.
+            training_data: Dictionary of `SupervisedDataset`.
+            fidelity_features: Index of fidelity parameter as input columns.
         """
-        fidelity_features = kwargs.get("fidelity_features")
-        if fidelity_features is None:
-            raise ValueError(f"Fidelity features required for {cls.__name__}.")
+        if len(fidelity_features) != 1:
+            raise UnsupportedError("Multiple fidelity features not supported.")
 
-        return {
-            "train_X": training_data.X,
-            "train_Y": training_data.Y,
-            "data_fidelity": fidelity_features[0],
-        }
+        inputs = super().construct_inputs(training_data=training_data, **kwargs)
+        inputs["data_fidelity"] = fidelity_features[0]
+        return inputs
 
 
 class FixedNoiseMultiFidelityGP(FixedNoiseGP):
@@ -235,27 +235,24 @@ class FixedNoiseMultiFidelityGP(FixedNoiseGP):
         self.to(train_X)
 
     @classmethod
-    def construct_inputs(cls, training_data: TrainingData, **kwargs) -> Dict[str, Any]:
-        r"""Construct kwargs for the `Model` from `TrainingData` and other options.
+    def construct_inputs(
+        cls,
+        training_data: SupervisedDataset,
+        fidelity_features: List[int],
+        **kwargs,
+    ) -> Dict[str, Any]:
+        r"""Construct `Model` keyword arguments from a dict of `SupervisedDataset`.
 
         Args:
-            training_data: `TrainingData` container with data for single outcome
-                or for multiple outcomes for batched multi-output case.
-            **kwargs: Options, expected for this class:
-                - fidelity_features: List of columns of X that are fidelity parameters.
+            training_data: Dictionary of `SupervisedDataset`.
+            fidelity_features: Column indices of fidelity features.
         """
-        fidelity_features = kwargs.get("fidelity_features")
-        if fidelity_features is None:
-            raise ValueError(f"Fidelity features required for {cls.__name__}.")
-        if training_data.Yvar is None:
-            raise ValueError(f"Yvar required for {cls.__name__}.")
+        if len(fidelity_features) != 1:
+            raise UnsupportedError("Multiple fidelity features not supported.")
 
-        return {
-            "train_X": training_data.X,
-            "train_Y": training_data.Y,
-            "train_Yvar": training_data.Yvar,
-            "data_fidelity": fidelity_features[0],
-        }
+        inputs = super().construct_inputs(training_data=training_data, **kwargs)
+        inputs["data_fidelity"] = fidelity_features[0]
+        return inputs
 
 
 def _setup_multifidelity_covar_module(
