@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -16,9 +16,10 @@ from __future__ import annotations
 
 import warnings
 from contextlib import ExitStack
-from typing import Any, List, Optional, Union, Tuple
+from typing import Any, List, Optional, Tuple, Union
 
 import torch
+from botorch.acquisition.objective import PosteriorTransform
 from botorch.models.gpytorch import BatchedMultiOutputGPyTorchModel
 from botorch.models.transforms.input import InputTransform
 from botorch.models.transforms.outcome import OutcomeTransform, Standardize
@@ -38,10 +39,7 @@ from gpytorch.lazy import (
     LazyTensor,
     ZeroLazyTensor,
 )
-from gpytorch.likelihoods import (
-    GaussianLikelihood,
-    Likelihood,
-)
+from gpytorch.likelihoods import GaussianLikelihood, Likelihood
 from gpytorch.models import ExactGP
 from gpytorch.priors.torch_priors import GammaPrior, MultivariateNormalPrior
 from gpytorch.settings import fast_pred_var, skip_posterior_variances
@@ -409,9 +407,18 @@ class HigherOrderGP(BatchedMultiOutputGPyTorchModel, ExactGP):
         X: Tensor,
         output_indices: Optional[List[int]] = None,
         observation_noise: Union[bool, Tensor] = False,
+        posterior_transform: Optional[PosteriorTransform] = None,
         **kwargs: Any,
     ) -> GPyTorchPosterior:
         self.eval()  # make sure we're calling a posterior
+
+        if posterior_transform is not None:
+            # this could be very costly, disallow for now
+            raise NotImplementedError(
+                "Posterior transforms currently not supported for "
+                f"{self.__class__.__name__}"
+            )
+
         # input transforms are applied at `posterior` in `eval` mode, and at
         # `model.forward()` at the training time
         X = self.transform_inputs(X)
@@ -502,7 +509,6 @@ class HigherOrderGP(BatchedMultiOutputGPyTorchModel, ExactGP):
             )
             if hasattr(self, "outcome_transform"):
                 posterior = self.outcome_transform.untransform_posterior(posterior)
-
             return posterior
 
     def make_posterior_variances(self, joint_covariance_matrix: LazyTensor) -> Tensor:
