@@ -10,6 +10,14 @@ model outputs. Outcome transformations are typically part of a Model and
 applied (i) within the model constructor to transform the train observations
 to the model space, and (ii) in the `Model.posterior` call to untransform
 the model posterior back to the original space.
+
+References
+
+.. [eriksson2021scalable]
+    D. Eriksson, M. Poloczek. Scalable Constrained Bayesian Optimization.
+    International Conference on Artificial Intelligence and Statistics. PMLR, 2021,
+    http://proceedings.mlr.press/v130/eriksson21a.html
+
 """
 
 from __future__ import annotations
@@ -25,7 +33,7 @@ from botorch.models.transforms.utils import (
 )
 from botorch.posteriors import GPyTorchPosterior, Posterior, TransformedPosterior
 from botorch.utils.transforms import normalize_indices
-from gpytorch.lazy import CholLazyTensor, DiagLazyTensor
+from gpytorch.lazy import BlockDiagLazyTensor, CholLazyTensor, DiagLazyTensor
 from torch import Tensor
 from torch.nn import Module, ModuleDict
 
@@ -369,9 +377,20 @@ class Standardize(OutcomeTransform):
             covar_tf = CholLazyTensor(mvn.scale_tril * scale_fac.unsqueeze(-1))
         else:
             lcv = mvn.lazy_covariance_matrix
-            # allow batch-evaluation of the model
-            scale_mat = DiagLazyTensor(scale_fac.expand(lcv.shape[:-1]))
-            covar_tf = scale_mat @ lcv @ scale_mat
+            scale_fac = scale_fac.expand(lcv.shape[:-1])
+            # TODO: Remove the custom logic with next GPyTorch release (T126095032).
+            if isinstance(lcv, BlockDiagLazyTensor):
+                # Keep the block diag structure of lcv.
+                base_lcv = lcv.base_lazy_tensor
+                scale_mat = DiagLazyTensor(
+                    scale_fac.view(*scale_fac.shape[:-1], lcv.num_blocks, -1)
+                )
+                base_lcv_tf = scale_mat @ base_lcv @ scale_mat
+                covar_tf = BlockDiagLazyTensor(base_lazy_tensor=base_lcv_tf)
+            else:
+                # allow batch-evaluation of the model
+                scale_mat = DiagLazyTensor(scale_fac)
+                covar_tf = scale_mat @ lcv @ scale_mat
 
         kwargs = {"interleaved": mvn._interleaved} if posterior._is_mt else {}
         mvn_tf = mvn.__class__(mean=mean_tf, covariance_matrix=covar_tf, **kwargs)
@@ -631,3 +650,127 @@ class Power(OutcomeTransform):
             posterior=posterior,
             sample_transform=lambda x: x.pow(1.0 / self.power),
         )
+
+
+class Bilog(OutcomeTransform):
+    r"""Bilog-transform outcomes.
+
+    The Bilog transform [eriksson2021scalable]_ is useful for modeling outcome
+    constraints as it magnifies values near zero and flattens extreme values.
+    """
+
+    def __init__(self, outputs: Optional[List[int]] = None) -> None:
+        r"""Bilog-transform outcomes.
+
+        Args:
+            outputs: Which of the outputs to Bilog-transform. If omitted, all
+                outputs will be transformed.
+        """
+        super().__init__()
+        self._outputs = outputs
+
+    def subset_output(self, idcs: List[int]) -> OutcomeTransform:
+        r"""Subset the transform along the output dimension.
+
+        Args:
+            idcs: The output indices to subset the transform to.
+
+        Returns:
+            The current outcome transform, subset to the specified output indices.
+        """
+        new_outputs = None
+        if self._outputs is not None:
+            if min(self._outputs + idcs) < 0:
+                raise NotImplementedError(
+                    f"Negative indexing not supported for {self.__class__.__name__} "
+                    "when subsetting outputs and only transforming some outputs."
+                )
+            new_outputs = [i for i in self._outputs if i in idcs]
+        new_tf = self.__class__(outputs=new_outputs)
+        if not self.training:
+            new_tf.eval()
+        return new_tf
+
+    def forward(
+        self, Y: Tensor, Yvar: Optional[Tensor] = None
+    ) -> Tuple[Tensor, Optional[Tensor]]:
+        r"""Bilog-transform outcomes.
+
+        Args:
+            Y: A `batch_shape x n x m`-dim tensor of training targets.
+            Yvar: A `batch_shape x n x m`-dim tensor of observation noises
+                associated with the training targets (if applicable).
+
+        Returns:
+            A two-tuple with the transformed outcomes:
+
+            - The transformed outcome observations.
+            - The transformed observation noise (if applicable).
+        """
+        Y_tf = Y.sign() * (Y.abs() + 1.0).log()
+        outputs = normalize_indices(self._outputs, d=Y.size(-1))
+        if outputs is not None:
+            Y_tf = torch.stack(
+                [
+                    Y_tf[..., i] if i in outputs else Y[..., i]
+                    for i in range(Y.size(-1))
+                ],
+                dim=-1,
+            )
+        if Yvar is not None:
+            raise NotImplementedError(
+                "Bilog does not yet support transforming observation noise"
+            )
+        return Y_tf, Yvar
+
+    def untransform(
+        self, Y: Tensor, Yvar: Optional[Tensor] = None
+    ) -> Tuple[Tensor, Optional[Tensor]]:
+        r"""Un-transform bilog-transformed outcomes
+
+        Args:
+            Y: A `batch_shape x n x m`-dim tensor of bilog-transfomred targets.
+            Yvar: A `batch_shape x n x m`-dim tensor of bilog-transformed
+                observation noises associated with the training targets
+                (if applicable).
+
+        Returns:
+            A two-tuple with the un-transformed outcomes:
+
+            - The un-transformed outcome observations.
+            - The un-transformed observation noise (if applicable).
+        """
+        Y_utf = Y.sign() * (Y.abs().exp() - 1.0)
+        outputs = normalize_indices(self._outputs, d=Y.size(-1))
+        if outputs is not None:
+            Y_utf = torch.stack(
+                [
+                    Y_utf[..., i] if i in outputs else Y[..., i]
+                    for i in range(Y.size(-1))
+                ],
+                dim=-1,
+            )
+        if Yvar is not None:
+            # TODO: Delta method, possibly issue warning
+            raise NotImplementedError(
+                "Bilog does not yet support transforming observation noise"
+            )
+        return Y_utf, Yvar
+
+    def untransform_posterior(self, posterior: Posterior) -> Posterior:
+        r"""Un-transform the bilog-transformed posterior.
+
+        Args:
+            posterior: A posterior in the bilog-transformed space.
+
+        Returns:
+            The un-transformed posterior.
+        """
+        if self._outputs is not None:
+            raise NotImplementedError(
+                "Bilog does not yet support output selection for untransform_posterior"
+            )
+        return TransformedPosterior(
+            posterior=posterior,
+            sample_transform=lambda x: x.sign() * (x.abs().exp() - 1.0),
+        )
