@@ -98,6 +98,11 @@ def gen_batch_initial_conditions(
         >>>     qEI, bounds, q=3, num_restarts=25, raw_samples=500
         >>> )
     """
+    if bounds.isinf().any():
+        raise NotImplementedError(
+            "Currently only finite values in `bounds` are supported "
+            "for generating initial conditions for optimization."
+        )
     options = options or {}
     seed: Optional[int] = options.get("seed")
     batch_limit: Optional[int] = options.get(
@@ -293,6 +298,7 @@ def gen_one_shot_kg_initial_conditions(
     value_function = _get_value_function(
         model=acq_function.model,
         objective=acq_function.objective,
+        posterior_transform=acq_function.posterior_transform,
         sampler=acq_function.inner_sampler,
         project=getattr(acq_function, "project", None),
     )
@@ -682,8 +688,9 @@ def sample_points_around_best(
             # the view() is to ensure that best_idcs is not a scalar tensor
             best_idcs = torch.topk(f_pred, n_best).indices.view(-1)
             best_X = X[best_idcs]
+    use_perturbed_sampling = best_X.shape[-1] >= 20 or prob_perturb is not None
     n_trunc_normal_points = (
-        n_discrete_points // 2 if best_X.shape[-1] >= 20 else n_discrete_points
+        n_discrete_points // 2 if use_perturbed_sampling else n_discrete_points
     )
     perturbed_X = sample_truncated_normal_perturbations(
         X=best_X,
@@ -691,7 +698,7 @@ def sample_points_around_best(
         sigma=sigma,
         bounds=bounds,
     )
-    if best_X.shape[-1] > 20 or prob_perturb is not None:
+    if use_perturbed_sampling:
         perturbed_subset_dims_X = sample_perturbed_subset_dims(
             X=best_X,
             bounds=bounds,
